@@ -60,7 +60,7 @@ class Animation extends St.Bin {
     }
 
     _loadFile(file, width, height) {
-        let [validResourceScale, resourceScale] = this.get_resource_scale();
+        const resourceScale = this.get_resource_scale();
         let wasPlaying = this._isPlaying;
 
         if (this._isPlaying)
@@ -69,12 +69,6 @@ class Animation extends St.Bin {
         this._isLoaded = false;
         this.destroy_all_children();
 
-        if (!validResourceScale) {
-            if (wasPlaying)
-                this.play();
-            return;
-        }
-
         let textureCache = St.TextureCache.get_default();
         let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
         this._animations = textureCache.load_sliced_image(file, width, height,
