@@ -21,8 +21,7 @@ const WorkspaceAnimation = imports.ui.workspaceAnimation;
 const { loadInterfaceXML } = imports.misc.fileUtils;
 
 var SHELL_KEYBINDINGS_SCHEMA = 'org.gnome.shell.keybindings';
-var MINIMIZE_WINDOW_ANIMATION_TIME = 400;
-var MINIMIZE_WINDOW_ANIMATION_MODE = Clutter.AnimationMode.EASE_OUT_EXPO;
+var MINIMIZE_WINDOW_ANIMATION_TIME = 200;
 var SHOW_WINDOW_ANIMATION_TIME = 150;
 var DIALOG_SHOW_WINDOW_ANIMATION_TIME = 100;
 var DESTROY_WINDOW_ANIMATION_TIME = 150;
@@ -954,7 +953,6 @@ var WindowManager = class {
 
         global.display.connect('notify::focus-window', updateUnfullscreenGesture);
         global.display.connect('in-fullscreen-changed', updateUnfullscreenGesture);
-        updateUnfullscreenGesture();
 
         global.stage.add_action(topDragAction);
 
@@ -1155,7 +1153,7 @@ var WindowManager = class {
             actor.ease({
                 opacity: 0,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
-                mode: MINIMIZE_WINDOW_ANIMATION_MODE,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
                 onStopped: () => this._minimizeWindowDone(shellwm, actor),
             });
         } else {
@@ -1186,7 +1184,7 @@ var WindowManager = class {
                 x: xDest,
                 y: yDest,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
-                mode: MINIMIZE_WINDOW_ANIMATION_MODE,
+                mode: Clutter.AnimationMode.EASE_IN_EXPO,
                 onStopped: () => this._minimizeWindowDone(shellwm, actor),
             });
         }
@@ -1220,7 +1218,7 @@ var WindowManager = class {
             actor.ease({
                 opacity: 255,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
-                mode: MINIMIZE_WINDOW_ANIMATION_MODE,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
                 onStopped: () => this._unminimizeWindowDone(shellwm, actor),
             });
         } else {
@@ -1242,7 +1240,7 @@ var WindowManager = class {
                 actor.set_scale(0, 0);
             }
 
-            let rect = actor.meta_window.get_buffer_rect();
+            let rect = actor.meta_window.get_frame_rect();
             let [xDest, yDest] = [rect.x, rect.y];
 
             actor.show();
@@ -1252,7 +1250,7 @@ var WindowManager = class {
                 x: xDest,
                 y: yDest,
                 duration: MINIMIZE_WINDOW_ANIMATION_TIME,
-                mode: MINIMIZE_WINDOW_ANIMATION_MODE,
+                mode: Clutter.AnimationMode.EASE_IN_EXPO,
                 onStopped: () => this._unminimizeWindowDone(shellwm, actor),
             });
         }
@@ -1285,7 +1283,7 @@ var WindowManager = class {
     _prepareAnimationInfo(shellwm, actor, oldFrameRect, _change) {
         // Position a clone of the window on top of the old position,
         // while actor updates are frozen.
-        let actorContent = Shell.util_get_content_for_window_actor(actor, oldFrameRect);
+        let actorContent = actor.paint_to_content(oldFrameRect);
         let actorClone = new St.Widget({ content: actorContent });
         actorClone.set_offscreen_redirect(Clutter.OffscreenRedirect.ALWAYS);
         actorClone.set_position(oldFrameRect.x, oldFrameRect.y);
@@ -1873,25 +1871,10 @@ var WindowManager = class {
         if (event.type() !== Clutter.EventType.SCROLL)
             return Clutter.EVENT_PROPAGATE;
 
-        if (event.is_pointer_emulated())
+        const direction = event.get_scroll_direction();
+        if (direction === Clutter.ScrollDirection.SMOOTH)
             return Clutter.EVENT_PROPAGATE;
 
-        let direction = event.get_scroll_direction();
-        if (direction === Clutter.ScrollDirection.SMOOTH) {
-            const [dx, dy] = event.get_scroll_delta();
-            if (Math.abs(dx) > Math.abs(dy)) {
-                direction = dx < 0
-                    ? Clutter.ScrollDirection.LEFT
-                    : Clutter.ScrollDirection.RIGHT;
-            } else if (Math.abs(dy) > Math.abs(dx)) {
-                direction = dy < 0
-                    ? Clutter.ScrollDirection.UP
-                    : Clutter.ScrollDirection.DOWN;
-            } else {
-                return Clutter.EVENT_PROPAGATE;
-            }
-        }
-
         const workspaceManager = global.workspace_manager;
         const vertical = workspaceManager.layout_rows === -1;
         const rtl = Clutter.get_default_text_direction() === Clutter.TextDirection.RTL;
