@@ -390,17 +390,6 @@ scroll_bar_update_positions (StScrollBar *bar)
   scroll_bar_allocate_children (bar, &box);
 }
 
-static void
-st_scroll_bar_style_changed (StWidget *widget)
-{
-  StScrollBarPrivate *priv = ST_SCROLL_BAR_PRIVATE (widget);
-
-  st_widget_style_changed (ST_WIDGET (priv->trough));
-  st_widget_style_changed (ST_WIDGET (priv->handle));
-
-  ST_WIDGET_CLASS (st_scroll_bar_parent_class)->style_changed (widget);
-}
-
 static void
 bar_reactive_notify_cb (GObject    *gobject,
                         GParamSpec *arg1,
@@ -511,7 +500,6 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
 
   object_class->get_property = st_scroll_bar_get_property;
   object_class->set_property = st_scroll_bar_set_property;
@@ -524,8 +512,6 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
   actor_class->scroll_event   = st_scroll_bar_scroll_event;
   actor_class->unmap          = st_scroll_bar_unmap;
 
-  widget_class->style_changed = st_scroll_bar_style_changed;
-
   /**
    * StScrollBar:adjustment:
    *
